
from utils import MellHelper, unindent


def test_logic():

    expected_output = unindent(8, """
        Metadata:
        {
          "users": [
            {
              "name": "Diego",
              "email": "diego@company.com"
            },
            {
              "name": "Quelle",
              "email": "quelle@company.com"
            }
          ]
        }
        """)

    logic_name = 'add_email'
    logic_program = unindent(8, """
        def logic(args, meta):
            for user in meta.users:
                user.email = user.name.value.lower() + "@company.com"
        """)

    meta_name = 'users'
    meta_data = unindent(8, """
        {
          "users": [
            {
              "name":"Diego"
            },
            {
              "name":"Quelle"
            }
          ]
        }
        """)

    p = MellHelper('logic')
    p.create_project()
    p.create_metadata(meta_name, meta_data)
    p.create_logic(logic_name, logic_program)

    status, stdout, stderr = p.exec(f'--root {p.root_path} --show-metadata users')

    assert status == 0
    assert stderr == ''
    assert stdout == expected_output



from utils import MellHelper, unindent

def test_plugin():

    metaname = 'data'

    metadata = unindent(8, """
        {
            "projects":[
                {
                    "name": "Thoculi",
                    "cost_in_usd": "1000000.00",
                    "duration_in_months": 3
                },
                {
                    "name": "Catalina",
                    "cost_in_usd": "30000000.00",
                    "duration_in_months": 24
                },
                {
                    "name": "Gold",
                    "cost_in_usd": "2000000.00",
                    "duration_in_months": 2
                }
            ]
        }
        """)
    
    asset_template = unindent(8, """
        Project '|= meta.name =|' will last |= meta.duration_in_months =| month(s) and cost USD |= meta.cost_in_usd =|.
        """)
    
    plugin_program = unindent(8, """
        def plugin(args, meta, inflater):
            for i, project in enumerate(meta.projects):
                inflater.inflate('project.txt', project, to_file=f'project_{i}.txt')
        """)

    project_0 = unindent(8, """
        Project 'Thoculi' will last 3 month(s) and cost USD 1000000.00.""")

    project_1 = unindent(8, """
        Project 'Catalina' will last 24 month(s) and cost USD 30000000.00.""")

    project_2 = unindent(8, """
        Project 'Gold' will last 2 month(s) and cost USD 2000000.00.""")

    p = MellHelper("plugin")
    p.create_project()
    p.create_metadata(metaname, metadata)
    p.create_plugin('projects', plugin_program)
    p.create_asset('project.txt', asset_template)
    status, stdout, stderr = p.exec(f'--root {p.root_path} {metaname}')

    assert status == 0
    assert stderr == ''
    assert stdout == ''

    assert p.read_generated_file('project_0.txt') == project_0
    assert p.read_generated_file('project_1.txt') == project_1
    assert p.read_generated_file('project_2.txt') == project_2




from utils import MellHelper, unindent


def test_load_metadata():

    expected_output = unindent(8, """
        Metadata:
        {
          "user": {
            "name": "Diego Souza",
            "age": 33,
            "backpack": [
              "book",
              "pen",
              "bottle",
              "phone",
              "camera"
            ]
          }
        }
        """)

    meta_name = "data"

    meta_data = unindent(8, """
        {
          "user": {
            "name": "Diego Souza",
            "age": 33, 
            "backpack": [
              "book", 
              "pen", 
              "bottle", 
              "phone", 
              "camera"
            ]
          }
        }
        """)

    p = MellHelper('load_metadata')
    p.create_project()
    p.create_metadata(meta_name, meta_data)
    
    status, stdout, stderr = p.exec(f'--root {p.root_path} --show-metadata {meta_name}')

    assert status == 0
    assert stderr == ''
    assert stdout == expected_output

def test_metadata_set():

    expected_output = unindent(8, """
        Metadata:
        {
          "user": {
            "name": "Diego Souza",
            "age": "18",
            "backpack": [
              "book",
              "keys",
              "bottle",
              "phone",
              "camera"
            ]
          },
          "extra": "more"
        }
        """)

    meta_name = 'data'

    meta_data = unindent(8, """
        {
          "user": {
            "name": "Diego Souza",
            "age": 33, 
            "backpack": [
              "book", 
              "pen", 
              "bottle", 
              "phone", 
              "camera"
            ]
          }
        }
        """)

    p = MellHelper('metadata_set')
    p.create_project()
    p.create_metadata(meta_name, meta_data)
    
    status, stdout, stderr = p.exec(f'--root {p.root_path} --set "extra" "more" --set user.age 18 --set "user.backpack[1]" "keys" --show-metadata {meta_name}')

    assert status == 0
    assert stderr == ''
    assert stdout == expected_output




from utils import MellHelper, file_count


def test_new_root():

    p = MellHelper('new_root')
    
    p.delete()

    returncode, _, _ = p.exec(f'--new {p.root_path}')
    
    assert returncode == 0

    assert file_count(p.root_path    ) == 3

    assert file_count(p.style_path   ) == 5
    assert file_count(p.meta_path    ) == 0
    assert file_count(p.generate_path) == 0

    assert file_count(p.template_path) == 0
    assert file_count(p.asset_path   ) == 0
    assert file_count(p.logic_path   ) == 0
    assert file_count(p.plugin_path  ) == 0
    assert file_count(p.static_path  ) == 0

def test_new_plugin():

    p = MellHelper("new_plugin")
    name = 'new_plugin'

    p.delete()
    
    assert p.exec(f'--new {p.root_path}')[0] == 0
    assert p.exec(f'--style {p.style_path} --new-plugin {name}')[0] == 0
    assert file_count(p.plugin_path) == 1

def test_new_logic():
    
    p = MellHelper("new_logic")
    name = 'new_logic'

    p.delete()
    
    assert p.exec(f'--new {p.root_path}')[0] == 0
    assert p.exec(f'--style {p.style_path} --new-logic {name}')[0] == 0
    assert file_count(p.logic_path) == 1

def test_new_style():

    p = MellHelper("new_style")
    name = 'new_style2'

    p.delete()
    
    assert p.exec(f'--new {p.root_path}')[0] == 0
    assert p.exec(f'--root {p.root_path} --new-style {name}')[0] == 0

    p.set_style(name)
    
    assert file_count(p.root_path    ) == 4

    assert file_count(p.style_path   ) == 5
    assert file_count(p.meta_path    ) == 0
    assert file_count(p.generate_path) == 0

    assert file_count(p.template_path) == 0
    assert file_count(p.asset_path   ) == 0
    assert file_count(p.logic_path   ) == 0
    assert file_count(p.plugin_path  ) == 0
    assert file_count(p.static_path  ) == 0

